using System;
using System.Reflection;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using System.Xml.Serialization;

namespace SAMS.SharePoint.Tools.ListLibrary
{
    [XmlRoot(ElementName = "listitems", 
        Namespace = "http://schemas.microsoft.com/sharepoint/soap/")]
    public class ListItems<T> where T : StandardListItem
    {
        private RowHeader<T> _rowHeader = null;        

        [XmlElement(ElementName = "data", 
            Namespace = "urn:schemas-microsoft-com:rowset")]
        public RowHeader<T> RowHeader
        {
            get
            {
                return _rowHeader;
            }
            set
            {
                _rowHeader = value;
            }
        }

        public XmlNode GetBatchUpdateNode()
        {
            XmlDocument doc = new XmlDocument();
            XmlElement batchElement = doc.CreateElement("Batch");
            batchElement.SetAttribute("OnError", "Continue");
            
            doc.AppendChild(batchElement);
            int methodCount = 0;
            foreach (T item in _rowHeader.ListItems)
            {
                if (item.BatchMode != BatchMode.None)
                {
                    XmlElement method = doc.CreateElement("Method");
                    method.SetAttribute("ID", methodCount.ToString());
                    string batchMode;
                    switch (item.BatchMode)
                    {
                        case BatchMode.Delete:
                            batchMode = "Delete";
                            break;
                        case BatchMode.New:
                            batchMode = "New";
                            break;
                        default:
                            batchMode = "Update";
                            break;
                    }
                    method.SetAttribute("Cmd", batchMode);
                    batchElement.AppendChild(method);
                    // add fields to batch element
                    if (item.BatchMode != BatchMode.Delete)
                    {
                        foreach (PropertyInfo pi in typeof(T).GetProperties(BindingFlags.Public | BindingFlags.Instance))
                        {
                            // get the inbound field name of the property, if any. 
                            // if there is no inbound field name, it doesn't go in the update.
                            SharePointListFieldAttribute[] attributes =
                                (SharePointListFieldAttribute[])pi.GetCustomAttributes(typeof(SharePointListFieldAttribute), true);
                            if (attributes.Length > 0)
                            {
                                XmlElement fieldNode = doc.CreateElement("Field");
                                fieldNode.SetAttribute("Name", attributes[0].FieldName);
                                object val = pi.GetValue(item, null);
                                if (val != null)
                                    fieldNode.InnerText = val.ToString();

                                method.AppendChild(fieldNode);
                            }
                        }
                    }
                    else
                    {
                        // delete batchmode only gets 1 field: "ID".
                    }
                    methodCount++;
                }
            }
            return batchElement;
        }

        public static int GetIDFromLookupValue(string lookupValue)
        {
            string[] arr = lookupValue.Split(';');
            return Int32.Parse(arr[0]);
        }

        public static string GetDescriptionFromLookupValue(string lookupValue)
        {
            string[] arr = lookupValue.Split(';');
            return arr[1].Remove(0, 1); // remove the #-sign
        }

        public void TrackChanges()
        {
            foreach (T item in _rowHeader.ListItems)
            {
                item.PropertyChanged += new System.ComponentModel.PropertyChangedEventHandler(item_PropertyChanged);
            }
        }

        void item_PropertyChanged(object sender, System.ComponentModel.PropertyChangedEventArgs e)
        {
            ((StandardListItem)sender).BatchMode = BatchMode.Update;
        }

        public static ListItems<T> FromXml(string xmlNode)
        {
            XmlSerializer xs = new XmlSerializer(typeof(ListItems<T>));
            XmlTextReader xtr = new XmlTextReader(xmlNode, XmlNodeType.Element, null);
            object o = xs.Deserialize(xtr);
            xtr.Close();
            return (ListItems<T>)o;
        }
    }
}


